/*



FastPACS
2006-2013 by Precision Diagnostic Systems, Inc.
Maple Park, IL 60151 USA




*/
//========================================================================================
	//==============================
	// CONSTANTS
	//==============================
	const int kMenuBarHeight = 72;
	const int kInfoBarHeight = 30;
	const int kStripWidth = 32;

	//==============================
	//
	//==============================
	TextWriter Writer;

	//==============================
	//
	//==============================
	String window_width() {
		switch (m_browser.type) {
		case "NN":
		case "GE":
		case "OP":
			return "window.innerWidth";
		case "IE":
			return "window.document.body.clientWidth";
		default:
			return "600";
		}
	}

	//==============================
	//
	//==============================
	String window_height() {
		switch (m_browser.type) {
		case "NN":
		case "GE":
		case "OP":
			return "window.innerHeight";
		case "IE":
			return "window.document.body.clientHeight";
		default:
			return "400";
		}
	}

	//==============================
	//
	//
	//
	//==============================
	class ImageSpaceObject {
		public Double x_max = Double.MinValue, x_min = Double.MaxValue;
		public Double y_max = Double.MinValue, y_min = Double.MaxValue;
		public Double z_max = Double.MinValue, z_min = Double.MaxValue;
		public Double xc = 0, yc = 0, zc = 0;

		public Boolean same_voi_value = false, has_valid_voi = false;
		public Double wc = Double.NaN, ww = Double.NaN;

		public void calc_center() {
		 	this.xc = (this.x_max + this.x_min) / 2;
			this.yc = (this.y_max + this.y_min) / 2;
			this.zc = (this.z_max + this.z_min) / 2;
		}
	}

	//==============================
	//
	//
	//
	//==============================
	class ImageObjectComparer: IComparer {
      int IComparer.Compare( Object x, Object y )  {
          return (x as ImageObject).number - (y as ImageObject).number;
      }
	}


	//==============================
	//
	//
	//
	//==============================
	class ImageObject {
		public readonly String file;
		public readonly String uid;
		public readonly UInt32 rows, cols;
		public readonly Int32 number;
		public readonly UInt32 date;
		public readonly Double time;
		public readonly Double xs, ys;
		public readonly Double x, y, z;
		public readonly UInt32 xd, yd;
		public readonly Double wc, ww;
		public readonly UInt16 bits;
		public const String image_orientation_table = "  RLLRAPPAFHHF";
			// "", "RL", "LR", "AP", "PA", "FH", "HF";

		//==============================
		//
		//==============================
		public ImageObject(SOPInstanceObject file, ImageSpaceObject s) {

			this.file = file.path;

			String date_str = null;
			String time_str = null;
			String wc_str = null;
			String ww_str = null;
			String number_str = null;

			this.number = -1;

			ArrayList position = null;
			ArrayList orientation = null;
			ArrayList spacing = null;

			foreach (DataElementObject deo in file.data) {
				if (deo.vm == 0 || deo.value == null)
					continue;
				UInt32 g = deo.tag & 0xFFFF0000;
				UInt32 e = deo.tag & 0x0000FFFF;
				if (g == 0x00080000) {
					if (e == 0x0018) {
						this.uid = (String)deo.value;
					} else if (e == 0x0023) {
						date_str = (String)deo.value;
					} else if (e == 0x0033) {
						time_str = (String)deo.value;
					}
					continue;
				} else if (g == 0x00200000) {
					if (e == 0x0013) {
						number_str = (String)deo.value;
					} else if (e == 0x0032) {
						if (deo.vm == 3)
							position = (ArrayList)deo.value;
					} else if (e == 0x0037) {
						if (deo.vm == 6)
							orientation = (ArrayList)deo.value;
					}
					continue;
				} else if (g == 0x00280000) {
					if (e == 0x0010) {
						this.rows = (UInt16)deo.value;
					} else if (e == 0x0011) {
						this.cols = (UInt16)deo.value;
					} else if (e == 0x0030) {
						if (deo.vm == 2)
							spacing = (ArrayList)deo.value;
					} else if (e == 0x0100) {
						this.bits = (UInt16)deo.value;
					} else if (e == 0x1050) {
						wc_str = (String)get_data(deo, VR.DS);
					} else if (e == 0x1051) {
						ww_str = (String)get_data(deo, VR.DS);
					}
					continue;
				} else if (deo.tag > 0x00281051) {
					break;
				}
			}

			if (date_str != null) {
				try {
					date_str = normalizeDate(date_str);
					this.date = UInt32.Parse(date_str);
				} catch (FormatException ex) {
					// prevent to throw the exception to caller
				}
			}

			if (time_str != null) {
				try {
					time_str = normalizeTime(time_str);
					this.time = Double.Parse(time_str);
				} catch (FormatException ex) {
					// prevent to throw the exception to caller
				}
			}

			if (number_str != null) {
				try {
					this.number = Int32.Parse(number_str, IS_ns);
				} catch (FormatException ex) {
					// prevent to throw the exception to caller
				}
			}

			if (wc_str != null && ww_str != null) {
				Double twc, tww;
				try {
					twc = Double.Parse(wc_str, DS_ns);
					tww = Double.Parse(ww_str, DS_ns);
					if (!Double.IsNaN(twc) && !Double.IsNaN(tww)) {
						this.wc = twc;
						this.ww = tww;

						if (Double.IsNaN(s.wc) || Double.IsNaN(s.ww)) {
							s.wc = this.wc;
							s.ww = this.ww;
							s.has_valid_voi = true;
							s.same_voi_value = true;
						} else {
							if (s.same_voi_value) {
								if (s.wc != this.wc || s.ww != this.ww)
									s.same_voi_value = false;
							}
						}
					}
				} catch (FormatException ex) {
					// prevent to throw the exception to caller
				}
			}

			if (spacing != null) {
				try {
					this.xs = Double.Parse((String)spacing[1], DS_ns);
					this.ys = Double.Parse((String)spacing[0], DS_ns);
				} catch (FormatException ex) {
					// prevent to throw the exception to caller
				}
			}

			if (position != null && orientation != null && spacing != null) {
				try {
					Double x0 = Double.Parse((String)position[0], DS_ns);
					Double y0 = Double.Parse((String)position[1], DS_ns);
					Double z0 = Double.Parse((String)position[2], DS_ns);
					Double cos_col_x = Double.Parse((String)orientation[0], DS_ns);
					Double cos_col_y = Double.Parse((String)orientation[1], DS_ns);
					Double cos_col_z = Double.Parse((String)orientation[2], DS_ns);
					Double cos_row_x = Double.Parse((String)orientation[3], DS_ns);
					Double cos_row_y = Double.Parse((String)orientation[4], DS_ns);
					Double cos_row_z = Double.Parse((String)orientation[5], DS_ns);

					this.x = x0 + (cols / 2) * this.xs * cos_col_x + (rows / 2) * this.ys * cos_row_x;
					this.y = y0 + (cols / 2) * this.xs * cos_col_y + (rows / 2) * this.ys * cos_row_y;
					this.z = z0 + (cols / 2) * this.xs * cos_col_z + (rows / 2) * this.ys * cos_row_z;

					if (this.x > s.x_max)
						s.x_max = this.x;
					if (this.x < s.x_min)
						s.x_min = this.x;

					if (this.y > s.y_max)
						s.y_max = this.y;
					if (this.y < s.y_min)
						s.y_min = this.y;

					if (this.z > s.z_max)
						s.z_max = this.z;
					if (this.z < s.z_min)
						s.z_min = this.z;

					// "", "RL", "LR", "AP", "PA", "FH", "HF";

					if (cos_col_x > 0.866)
						this.xd = 1;
					else if (cos_col_x < -0.866)
						this.xd = 2;
					else if (cos_col_y > 0.866)
						this.xd = 3;
					else if (cos_col_y < -0.866)
						this.xd = 4;
					else if (cos_col_z > 0.866)
						this.xd = 5;
					else if (cos_col_z < -0.866)
						this.xd = 6;
					else
						this.xd = 0;

					if (cos_row_x > 0.866)
						this.yd = 1;
					else if (cos_row_x < -0.866)
						this.yd = 2;
					else if (cos_row_y > 0.866)
						this.yd = 3;
					else if (cos_row_y < -0.866)
						this.yd = 4;
					else if (cos_row_z > 0.866)
						this.yd = 5;
					else if (cos_row_z < -0.866)
						this.yd = 6;
					else
						this.yd = 0;

				} catch (FormatException ex) {
					// prevent to throw the exception to caller
				}

			}
		}

		//==============================
		//
		//==============================
		public String new_args(ImageSpaceObject s) {
			return "\"" + this.file + "\""
				+ "," + this.number
				+ "," + this.date
				+ "," + this.time
				+ "," + this.rows
				+ "," + this.cols
				+ "," + this.xs
				+ "," + this.ys
				+ "," + (this.x - s.xc).ToString()
				+ "," + (this.y - s.yc).ToString()
				+ "," + (this.z - s.zc).ToString()
				+ "," + this.xd
				+ "," + this.yd
				+ "," + this.wc
				+ "," + this.ww;
		}

		public static int compareNumber(ImageObject a, ImageObject b) {
			return a.number - b.number;
		}
		public static int compareDateTime(ImageObject a, ImageObject b) {
			if (a.date > b.date)
				return 1;
			else if (a.date < b.date)
				return -1;
			if (a.time > b.time)
				return 1;
			else if (a.time < b.time)
				return -1;
			return 0;
		}
		public static int compareX(ImageObject a, ImageObject b) {
			return (int)(a.x - b.x);
		}
		public static int compareY(ImageObject a, ImageObject b) {
			return (int)(a.y - b.y);
		}
		public static int compareZ(ImageObject a, ImageObject b) {
			return (int)(a.z - b.z);
		}

	}

	//==============================
	//
	//
	//
	//==============================
	class VOIEntry {
		public readonly String label;
		public readonly Int32 wc;
		public readonly Int32 ww;

		//==============================
		//
		//==============================
		public VOIEntry(Int32 wc, Int32 ww, String label) {
			this.wc = wc;
			this.ww = ww;
			this.label = label;
		}
	}

	//==============================
	//
	//
	//
	//==============================
	class VOITableObject {

		public readonly Int32 ww;
		public readonly Int32 wc;

		public readonly ArrayList table;

		//==============================
		//
		//==============================
		public VOITableObject(ImageSpaceObject s, DicomStudy dom, QueryStringValue qsv, ArrayList presetTable) {

			String modality = get_data_CS(null, dom, 0x00080060, null);
			UInt16 bitsStored = get_data_US(null, dom, 0x00280101, 0);

			this.table = new ArrayList();

			this.table.Add(new VOIEntry(-1, -1, "CUSTOM"));

			VOIEntry qsvoi = null;
			if (qsv.voi != null) {
				String[] cw = qsv.voi.Split(new Char[] {','});
				if (cw.Length == 2) {
					qsvoi = new VOIEntry(Int32.Parse(cw[0]), Int32.Parse(cw[1]), "");
				}
			}

			VOIEntry tagvoi = null;
			if (s.has_valid_voi) {
				Int32 wc, ww;
				// This was before
				/*
				if (s.same_voi_value) {
					wc = (Int32)s.wc;
					ww = (Int32)s.ww;
				} else {
					wc = ww = 0;
				}
				*/
				// And now
				wc = (Int32)s.wc;
				ww = (Int32)s.ww;
				tagvoi = new VOIEntry(wc, ww, "ORIGINAL");
				this.table.Add(tagvoi);
			}

			if (qsvoi != null) {
				this.wc = qsvoi.wc;
				this.ww = qsvoi.ww;
			} else if (tagvoi != null) {
				this.wc = tagvoi.wc;
				this.ww = tagvoi.ww;
			} else {
				Int32 c, w;
				if (modality == "CT") {
					c = 35;
					w = 250;
				} else if (modality == "CR" || modality == "MR") {
					c = 2048;
					w = 4096;
				} else {
					w = 1 >> (int)bitsStored;
					c = w / 2;
				}
				this.wc = c;
				this.ww = w;
			}

			foreach (Object voi in presetTable)
				this.table.Add(voi);
		}


		//==============================
		//
		//==============================
		public VOIEntry this[int index] {
			get { return (VOIEntry)table[index]; }
		}

	}

	//==============================
	//
	//
	//
	//==============================
	class MenuItem {
		public readonly String menu_value, menu_label;
		public MenuItem(String value, String label) {
			menu_value = value;
			menu_label = label;
		}
	}

	//==============================
	//
	//
	//
	//==============================
	abstract class MenuBase {

		public MenuItem this[int index] {
			get { return (MenuItem)table[index]; }
		}

		public Int32 Count {
			get { return table.Count; }
		}

		protected ArrayList table;
	}


	//==============================
	//
	//
	//
	//==============================
	class DrMenu {
		String label, sig, handler;
		UInt32 list_width;
		MenuBase menu;
		TextWriter Writer;
		BrowserCheck m_browser;

		//==============================
		//
		//==============================
		public DrMenu(TextWriter writer, BrowserCheck browser, String label, String sig, UInt32 list_width, MenuBase menu, String handler) {
			Writer = writer;
			m_browser = browser;
			this.label = label;
			this.sig = sig;
			this.list_width = list_width;
			this.menu = menu;
			this.handler = handler;
		}

		//==============================
		//
		//==============================
		public void print_menu_button() {
			if (m_browser.type == "NN")
				Writer.Write("<TD CLASS='MenuButton' ALIGN='CENTER' VALIGN='MIDDLE' NOWRAP><ILAYER ID='DrMenuAnchor-" + this.sig + "'><NOBR CLASS='BarText'>" + this.label + "</NOBR></ILAYER></TD>");
			else if (m_browser.type == "IE")
				Writer.Write("<TD CLASS='MenuButton' ALIGN='CENTER' VALIGN='MIDDLE' NOWRAP ID='DrMenuButton-" + this.sig + "' onmouseover='drmenu_event(event,true,\"" + this.sig + "\",true)' onmouseout='drmenu_event(event,false,\"" + this.sig + "\",true)'>" + this.label + "</TD>");
			else if (m_browser.type == "GE")
				Writer.Write("<TD CLASS='MenuButton' ALIGN='CENTER' VALIGN='MIDDLE' NOWRAP ID='DrMenuButton-" + this.sig + "' onmouseover='drmenu_event(event,true,\"" + this.sig + "\",true)' onmouseout='drmenu_event(event,false,\"" + this.sig + "\",true)'>" + this.label + "</TD>");
			else
				Writer.Write("<TD></TD>");

		}

		//==============================
		//
		//==============================
		void print_menu_items() {
			for (int i = 0; i < menu.Count; i++) {
				MenuItem item = menu[i];
				if (i != 0)
					Writer.Write("<BR>");
				Writer.Write("<NOBR CLASS='DrMenu-listItem'><A CLASS='DrMenu-item' ");
				if (m_browser.type == "NN" && m_browser.os == "M")
					Writer.Write("HREF='javascript:void()' onClick='drmenu_item_click(\"" + this.sig + "\"," + this.handler + ",\"" + item.menu_value + "\"); return false;'");
				else
					Writer.Write("HREF='javascript:drmenu_item_click(\"" + this.sig + "\"," + this.handler + ",\"" + item.menu_value + "\");'");
				Writer.Write(">" + item.menu_label + "</A></NOBR>");
			}
		}

		//==============================
		//
		//==============================
		public void print_menu_list() {
			if (m_browser.type == "NN") {
				Writer.Write("<LAYER ID='DrMenuButton-" + this.sig + "' Z-INDEX='4'></LAYER>\n");
				Writer.Write("<LAYER CLASS='DrMenu-list' ID='DrMenuList-" + this.sig + "' VISIBILITY='HIDDEN' Z-INDEX='4' WIDTH='" + this.list_width + "'>");
				print_menu_items();
				Writer.Write("</LAYER>\n");
			} else if (m_browser.type == "IE") {
				if (m_browser.os == "M") {
					Writer.Write("<DIV CLASS='DrMenu-list' ID='DrMenuList-" + this.sig + "' STYLE='width:" + this.list_width + ";' onmouseover='drmenu_event(event,true,\"" + this.sig + "\",false)' onmouseout='drmenu_event(event,false,\"" + this.sig + "\",false)'>");
					print_menu_items();
					Writer.Write("</DIV>\n");
				} else {
					Writer.Write("<DIV CLASS='DrMenu-list' ID='DrMenuList-" + this.sig + "' onmouseover='drmenu_event(event,true,\"" + this.sig + "\",false)' onmouseout='drmenu_event(event,false,\"" + this.sig + "\",false)'>");
					print_menu_items();
					Writer.Write("</DIV>\n");
				}
			} else if (m_browser.type == "GE") {
				Writer.Write("<DIV CLASS='DrMenu-list' ID='DrMenuList-" + this.sig + "' onmouseover='drmenu_event(event,true,\"" + this.sig + "\",false)' onmouseout='drmenu_event(event,false,\"" + this.sig + "\",false)'>");
				print_menu_items();
				Writer.Write("</DIV>\n");
			}
		}

	}

	//==========================
	//
	//==========================
	String dash(UInt32 dm, UInt32 dw) {
		UInt32 n = m_browser.os == "M" ? dm : dw;
		return new String('-', (Int32)n);
	}

	//==============================
	//
	//==============================
	void printVarText(String id, String label, UInt32 ww, UInt32 wm, UInt32 dw, UInt32 dm) {
		if (m_browser.type == "NN")
			Writer.Write("<TD CLASS='VarText' ALIGN='LEFT' VALIGN='MIDDLE' NOWRAP><NOBR CLASS='BarText'>" + label + "<ILAYER ID='" + id + "'><LAYER>" + dash(dm, dw) + "</LAYER></ILAYER></NOBR></TD>");
		else if (m_browser.type == "IE")
			Writer.Write("<TD CLASS='VarText' ALIGN='LEFT' VALIGN='MIDDLE' NOWRAP WIDTH='" + (m_browser.os == "M" ? wm : ww) + "'>" + label + "<SPAN ID='" + id + "'> </SPAN></TD>");
		else if (m_browser.type == "GE")
			Writer.Write("<TD CLASS='VarText' ALIGN='LEFT' VALIGN='MIDDLE' NOWRAP WIDTH='" + (m_browser.os == "M" ? wm : ww) + "'>" + label + "<SPAN ID='" + id + "'> </SPAN></TD>");
		else
			Writer.Write("<TD></TD>");
	}

	//==============================
	//
	//==============================
	void print_study_info(DicomStudy dom) {

		SOPInstanceObject file = (SOPInstanceObject)dom.fl[0];

		String PatientID = get_data_LO(file, null, 0x00100020, String.Empty);
		String PatientName = get_data_PN(file, null, 0x00100010, String.Empty);
		String StudyDate = get_data_DA(file, null, 0x00080020, "YYYYMMDD");
		String StudyTime = get_data_TM(file, null, 0x00080030, "hhmmss.x");
		String Modality = get_data_CS(file, null, 0x00080060, String.Empty);
		String StudyID = get_data_SH(file, null, 0x00200010, String.Empty);

		Writer.Write("EXAM: // ");
		Writer.Write(PatientID + ", " + PatientName);
		Writer.Write(" // ");
		Writer.Write(prettyDate(StudyDate) + ", " + prettyTime(StudyTime) + ", " + Modality + (StudyID != null ? ":" + StudyID : ""));
		Writer.Write(" // ");
	}

	//==========================
	//
	//==========================
	void print_desc_text(SOPInstanceObject file, UInt32 tag, String label, int vm) {

		ArrayList values = get_data_array(file, null, tag);
		if (values == null)
			return;

		Writer.Write("<NOBR>" + label + ": <FONT COLOR='white'>");
		if (vm == 0) {
			bool first = true;
			foreach (Object v in values) {
				if (first)
					first = false;
				else
					Writer.Write(", ");
				Writer.Write(v);
			}
		} else if (vm >= 1 && vm <= values.Count) {
			Object v = values[vm - 1];
			if (v != null)
				Writer.Write(v);
		}
		Writer.Write("</FONT></NOBR>; \n");
	}

//========================================================================================
